/*
 * UDP send/receive tool for packet loss detection.
 * Copyright 2007 by Toru Takahashi.
 */
package model;

import java.util.Set;
import java.util.TreeSet;
import java.util.prefs.Preferences;
import java.util.Collection;
import java.util.Arrays;

public abstract class UdpModel {

    public UdpModel(Class<? extends UdpModel> c) {
        preferences = Preferences.userNodeForPackage(c);
    }

    public void clear() throws Exception {
        preferences.clear();
    }

    protected void putToPreferences(String aKey, String aValue) {
        preferences.put(aKey, aValue);
    }

    protected Set<String> readStringSetFromPreferences(
        String aKey, String aDefaultValue
    ) {
        final String valueString = preferences.get(aKey, "");
        final String[] values = "".equals(valueString) ? 
            new String[0] : valueString.split(" ");
        Set<String> set = new TreeSet<String>(Arrays.asList(values));
        if (set.size() == 0) {
            set.add(aDefaultValue);
        }
        return set;
    }

    protected Set<Integer> readIntegerSetFromPreferences(
        String aKey, int aDefaultValue
    ) {
        final String valueString = preferences.get(aKey, "");
        final String[] valueStrings = valueString.split(" ");
        Set<Integer> set = new TreeSet<Integer>();
        for (String item : valueStrings) {
            try {
                set.add(Integer.valueOf(item));
            } catch (NumberFormatException ignore) {
            }
        }
        if (set.size() == 0) {
            set.add(aDefaultValue);
        }
        return set;
    }

    protected static String getStringForPreferences(Collection<?> aList) {
        StringBuilder sb = new StringBuilder();
        for (Object item : aList) {
            sb.append(item);
            sb.append(" ");
        }
        return sb.toString();
    }

    private Preferences preferences;
}
