/*
 * UDP send/receive tool for packet loss detection.
 * Copyright 2007 by Toru Takahashi.
 */
package receiver;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.GroupLayout;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JTextArea;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.LayoutStyle;
import javax.swing.JOptionPane;
import java.awt.event.ActionListener;
import java.awt.event.WindowListener;
import javax.swing.DefaultComboBoxModel;
import java.util.Vector;

/**
 * UdpReceivervOɂāAModel-View-Presenterp^[ۂ
 * ViewNXB
 * $Id$
 */
public class UdpReceiveFrame extends JFrame implements UdpReceiveView {

    public UdpReceiveFrame() {
        super("UDP Receive Program");
        setDefaultCloseOperation(EXIT_ON_CLOSE);

        JPanel panel = new JPanel();
        GroupLayout layout = new GroupLayout(panel);
        panel.setLayout(layout);
        layout.setAutoCreateGaps(true);
        layout.setAutoCreateContainerGaps(true);

        portLabel = new JLabel("M|[gF");
        portComboBox = new JComboBox();
        portComboBox.setEditable(true);
        sizeLabel = new JLabel("MTCYF");
        sizeComboBox = new JComboBox();
        sizeComboBox.setEditable(true);
        bufferLabel = new JLabel("obt@TCYF");
        bufferComboBox = new JComboBox();
        bufferComboBox.setEditable(true);
        countLabel = new JLabel("M񐔁F0");
        currentLabel = new JLabel("MIDF0/0");
        skippedPacketArea = new JTextArea();
        JScrollPane skippedPacketAreaPane = new JScrollPane(skippedPacketArea);
        receiveButton = new JButton("MJn");

        GroupLayout.ParallelGroup hGroup = layout.createParallelGroup();
        hGroup.addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup()
                                  .addComponent(portLabel)
                                  .addComponent(sizeLabel)
                                  .addComponent(bufferLabel)
                        )
                        .addGroup(layout.createParallelGroup()
                                  .addComponent(portComboBox)
                                  .addComponent(bufferComboBox)
                                  .addComponent(sizeComboBox)
                        )
                        .addComponent(receiveButton)
        );
        hGroup.addGroup(layout.createSequentialGroup()
                        .addComponent(countLabel)
                        .addPreferredGap(
                            LayoutStyle.ComponentPlacement.RELATED,
                            120, Short.MAX_VALUE
                        )
                        .addComponent(currentLabel)
        );
        hGroup.addComponent(skippedPacketAreaPane);
        layout.setHorizontalGroup(hGroup);

        GroupLayout.SequentialGroup vGroup = layout.createSequentialGroup();
        vGroup.addGroup(
            layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
            .addComponent(portLabel)
            .addComponent(portComboBox)
            .addComponent(receiveButton)
        );
        vGroup.addGroup(
            layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
            .addComponent(sizeLabel)
            .addComponent(sizeComboBox)
        );
        vGroup.addGroup(
            layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
            .addComponent(bufferLabel)
            .addComponent(bufferComboBox)
        );
        vGroup.addGroup(
            layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
            .addComponent(countLabel)
            .addComponent(currentLabel)
        );
        vGroup.addComponent(skippedPacketAreaPane);
        layout.setVerticalGroup(vGroup);

        add(panel);
    }

    @Override public void setPortList(Vector<Integer> aList) {
        portComboBox.setModel(new DefaultComboBoxModel(aList));
    }

    @Override public void setSizeList(Vector<Integer> aList) {
        sizeComboBox.setModel(new DefaultComboBoxModel(aList));
    }
    @Override public void setBufferList(Vector<Integer> aList) {
        bufferComboBox.setModel(new DefaultComboBoxModel(aList));
    }

    @Override public void setCount(final int aCount) {
        SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    countLabel.setText("M񐔁F" + aCount);
                }
            }
        );
    }

    @Override public void setCurrent(final int aCurrent, final int aTotalCount) {
        SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    currentLabel.setText(
                        "MIDF" + aCurrent + "/" + aTotalCount
                    );
                }
            }
        );
    }

    @Override public void putSkippedPacketId(int anId) {
        skippedPacketArea.append(anId + "\n");
    }

    @Override public void clearSkippedPacketId() {
        skippedPacketArea.setText(null);
    }

    @Override public int getPort() {
        Integer item = (Integer)portComboBox.getSelectedItem();
        return item;
    }

    @Override public int getDataSize() {
        Integer item = (Integer)sizeComboBox.getSelectedItem();
        return item;
    }

    @Override public int getBufferSize() {
        Integer item = (Integer)bufferComboBox.getSelectedItem();
        return item;
    }
    @Override public void addViewListener(WindowListener aListener) {
        this.addWindowListener(aListener);
    }

    @Override public void addReceiveListener(ActionListener aListener) {
        receiveButton.addActionListener(aListener);
    }

    @Override public void showView() {
        SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    UdpReceiveFrame.this.pack();
                    UdpReceiveFrame.this.setVisible(true);
                }
            }
        );
    }

    @Override public void showError(Exception e) {
        JOptionPane.showMessageDialog(
            this, e, "O", JOptionPane.ERROR_MESSAGE
        );
    }

    /**
     * View̃CAEgmF邽߂̒P̗pmain\bhB
     */
    public static final void main(final String[] args) {
        final UdpReceiveView view = new UdpReceiveFrame();
        view.showView();
    }

    private JLabel portLabel;
    private JComboBox portComboBox;
    private JLabel sizeLabel;
    private JComboBox sizeComboBox;
    private JLabel bufferLabel;
    private JComboBox bufferComboBox;
    private JLabel countLabel;
    private JLabel currentLabel;
    private JTextArea skippedPacketArea;
    private JButton receiveButton;
}
