/*
 * UDP send/receive tool for packet loss detection.
 * Copyright 2007 by Toru Takahashi.
 */
package receiver;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.SocketException;
import java.net.DatagramSocket;
import java.nio.ByteBuffer;
import java.net.DatagramPacket;
import java.io.IOException;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.awt.event.WindowAdapter;
import java.util.logging.Logger;
import java.awt.event.WindowEvent;
import java.util.concurrent.ExecutorService;

/**
 * UdpReceivervOɂāAModel-View-Presenterp^[ۂ
 * PresenterNXB
 * ܂AvONpmain\bh񋟂B
 * 
 * $Id$
 */
public class UdpReceivePresenter {
    /**
     * PresenteŕAViewModel̑oƊ֘Aێ̂ŁA̗҂
     * RXgN^B
     *
     * @param aView UdpReceiverViewCX^X
     * @param aModel UdpReceiverModelCX^X
     */
    public UdpReceivePresenter(UdpReceiveView aView, UdpReceiveModel aModel) {
        view = aView;
        model = aModel;

        executor = Executors.newSingleThreadExecutor();
        view.addReceiveListener(new ActionListener() {
                public void actionPerformed(ActionEvent ev) {
                    receiveCount = 0;
                    messageId = 0;
                    messageTotalCount = 0;
                    view.setCount(0);
                    view.setCurrent(0, 0);
                    view.clearSkippedPacketId();
                    executor.execute(new Runnable() {
                            public void run() {
                                try {
                                    receive();
                                } catch (Exception e) {
                                    e.printStackTrace();
                                    view.showError(e);
                                }
                            }
                        }
                    );
                }
            }
        );

        view.addViewListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    close();
                    model.save();
                }
            }
        );
        view.setPortList(model.getPortList());
        view.setSizeList(model.getDataSizeList());
        view.setBufferList(model.getBufferSizeList());
        view.showView();
    }

    /**
     * vONpmain\bhB
     *
     * R}hCƂĈȉB
     * <ul>
     * <li>-clean<br>
     *      Preferencesɕۑꂽ
     * <li>-nogui<br>
     *      ()ViewGUIgpȂ
     * </ul>
     */
   public static final void main(final String[] args) {
        for (int i=0; i<args.length; ++i) {
            if ("-clean".equals(args[i])) {
                cleanPreferences();
                return;
            }
        }
        UdpReceiveView view = new UdpReceiveFrame();
        UdpReceiveModel model = new UdpReceiveModel();
        UdpReceivePresenter presenter = new UdpReceivePresenter(view, model);
    }

    private static final void cleanPreferences() {
        System.out.println("Preferences폜܂B");
        UdpReceiveModel model = new UdpReceiveModel();
        try {
            model.clear();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void receive() throws SocketException, IOException {
        socket = new DatagramSocket(view.getPort());
        int receiveBufferSize = view.getBufferSize();
        socket.setReceiveBufferSize(receiveBufferSize);
        model.addBufferSize(receiveBufferSize);

        ByteBuffer buffer = ByteBuffer.allocate(view.getDataSize() + 1);
        byte[] array = buffer.array();
        DatagramPacket packet = new DatagramPacket(
            array, array.length
        );
        model.addPort(view.getPort());
        model.addDataSize(view.getDataSize());
        while (true) {
            socket.receive(packet);
            buffer.rewind();
            int previousMessageId = messageId;
            messageId = buffer.getInt();
            int messageTotalCount = buffer.getInt();
            LOGGER.finest("previousMessageId = " + previousMessageId +
                          ", messageId = " + messageId + 
                          ", messageTotalCount = " + messageTotalCount
            );

            if ( (previousMessageId + 1) != messageId) {
                while (previousMessageId++ < messageId) {
                    view.putSkippedPacketId(previousMessageId);
                }
            }
            view.setCurrent(messageId, messageTotalCount);
            view.setCount(++receiveCount);
        }
    }
    private void close() {
        if (socket != null) {
            socket.close();
        }
    }

    private UdpReceiveView view;
    private UdpReceiveModel model;
    private ExecutorService executor;
    private DatagramSocket socket;
    private int receiveCount;
    private int messageId;
    private int messageTotalCount;
    private static Logger LOGGER = Logger.getLogger(
        UdpReceivePresenter.class.getName()
    );
}
