/*
 * UDP send/receive tool for packet loss detection.
 * Copyright 2007 by Toru Takahashi.
 */
package sender;

import java.awt.event.ActionEvent;
import java.net.DatagramSocket;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.net.DatagramPacket;
import java.net.SocketException;
import java.io.IOException;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.AbstractAction;
import java.util.logging.Logger;
import java.util.concurrent.Executors;
import java.util.concurrent.Executor;

/**
 * UdpSendervOɂāAModel-View-Presenterp^[ۂ
 * PresenterNXB
 * ܂AvONpmain\bh񋟂B
 * 
 * $Id$
 */
public class UdpSendPresenter {

    /**
     * PresenteŕAViewModel̑oƊ֘Aێ̂ŁA̗҂
     * RXgN^B
     *
     * @param aView UdpSnderViewCX^X
     * @param aModel UdpSenderModelCX^X
     */
    public UdpSendPresenter(UdpSendView aView, UdpSendModel aModel) {
        view = aView;
        model = aModel;
        executor = Executors.newSingleThreadExecutor();

        view.addSendListener(new AbstractAction() {
                public void actionPerformed(ActionEvent ev) {
                    view.setSendEnabled(false);
                    final int count = view.getCount();
                    final int interval = view.getInterval();
                    executor.execute(new Runnable() {
                            public void run() {
                                try {                                    
                                    send(count, interval);
                                    view.showFinished();
                                } catch (Exception e) {
                                    e.printStackTrace();
                                    view.showError(e);
                                } finally {
                                    view.setSendEnabled(true);
                                }
                            }
                        }
                    );
                    model.addCount(count);
                    model.addInterval(interval);
                }
            }
        );

        view.addViewListener(new WindowAdapter() {
                public void windowClosing(WindowEvent ev) {
                    close();
                    model.save();
                }
            }
        );
        view.setAddressList(model.getAddressList());
        view.setSizeList(model.getSizeList());
        view.setPortList(model.getPortList());
        view.setIntervalList(model.getIntervalList());
        view.setCountList(model.getCountList());
        view.showView();
    }

    /**
     * vONpmain\bhB
     *
     * R}hCƂĈȉB
     * <ul>
     * <li>-clean<br>
     *      Preferencesɕۑꂽ
     * <li>-nogui<br>
     *      ()ViewGUIgpȂ
     * </ul>
     */
    public static final void main(final String[] args) {
        for (int i=0; i<args.length; ++i) {
            if ("-clean".equals(args[i])) {
                cleanPreferences();
                return;
            } else if ("-nogui".equals(args[i])) {
                
            }
        }
        UdpSendView view = new UdpSendFrame();
        UdpSendModel model = new UdpSendModel();
        UdpSendPresenter presenter = new UdpSendPresenter(view, model);
    }

    private static final void cleanPreferences() {
        System.out.println("Preferences폜܂B");
        UdpSendModel model = new UdpSendModel();
        try {
            model.clear();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void send(final int aCount, final int anInterval)
        throws SocketException, IOException, InterruptedException
    {
        LOGGER.entering(UdpSendPresenter.class.getName(), "send",
                        new Integer[] { aCount, anInterval });
        socket = new DatagramSocket();
        final InetSocketAddress address = getAddress();
        final ByteBuffer buffer = createBuffer();

        for (int i=1; i<=aCount; i++) {
            byte[] sendData = getSendData(buffer, i, aCount);
            DatagramPacket packet = new DatagramPacket(
                sendData, 0, sendData.length, address
            );
            socket.send(packet);
            if (anInterval != 0) {
                Thread.sleep(anInterval);
            }
        }
    }

    private InetSocketAddress getAddress() {
        InetSocketAddress address = new InetSocketAddress(
            view.getAddress(),
            view.getPort()
        );
        model.addAddress(view.getAddress());
        model.addPort(view.getPort());
        return address;
    }

    private ByteBuffer createBuffer() {
        int size = view.getDataSize();
        if (size < 8) {
            size = 8;
        }
        ByteBuffer buffer = ByteBuffer.allocate(size);
        buffer.position(8);
        for (int i=0; i<size - 8; i++) {
            buffer.put((byte)(i % 256));
        }
        model.addSize(size);
        return buffer;
    }

    private byte[] getSendData(ByteBuffer buffer, int i, int sum) {
        LOGGER.entering(UdpSendPresenter.class.getName(), "getSendData",
                        new Object[] { buffer, i, sum });
        buffer.rewind();
        buffer.putInt(i);
        buffer.putInt(sum);
        byte[] ret = buffer.array();
        LOGGER.exiting(UdpSendPresenter.class.getName(), "getSendData", ret);
        return ret;
    }

    private void close() {
        if (socket != null) {
            socket.close();
        }
    }

    private UdpSendView view;
    private UdpSendModel model;
    private DatagramSocket socket;
    private Executor executor;
    private static Logger LOGGER = Logger.getLogger(
        UdpSendPresenter.class.getName()
    );
}
